<?php
namespace App\Http\Controllers\Owner;

use App\Http\Controllers\Controller;

use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Http\Helpers\ResponseBuilder;
use App\Http\Helpers\ParamsValidator;
use App\Http\History\HistoryData;
use App\Http\History\HistoryTables;
use App\Modules\AdminRole\Application\AdminRoleCopier;
use App\Modules\AdminRole\Domain\IdAdminRole;
use App\Modules\AdminRole\Infraestructure\MysqlAdminRoleRepository;
use Illuminate\Support\Facades\DB;
use Intervention\Image\ImageManagerStatic as Image;

class c_Admin_Role extends Controller
{
	private $oDomain;
	/** @var HistoryData */
	private $history;

	public function __construct()
	{
		$this->oDomain = "Admin Role";
		$this->history = new HistoryData (HistoryTables::admin_role);
	}

	public function Insert(Request $Request){
		$oResponse  = array();
		$oValParams = array(    'AdminRole_Name'	=> 'required|string|max:250',
								'AdminRole_Abrv'	=> 'required|string|max:4',
								'AdminRole_Color'	=> 'required|string|max:10',
								'AdminRole_Admin'	=> 'required|int',
								'AdminRole_CalendarAccess'	=> 'required|int',
								'AdminRole_Status'	=> 'required|int'
							);


		// VALIDANDO DATOS
		$oResponse = ParamsValidator::Validate_Request($Request,$oValParams,$this->oDomain);
		
		//  INGRESANDO DATOS
		if ($oResponse["Response_Code"] == 200) {			
			$oParam = array(    Str::title( $Request->input("AdminRole_Name")),
								Str::upper( $Request->input("AdminRole_Abrv")),
								Str::upper( $Request->input("AdminRole_Color")),
											$Request->input("AdminRole_Admin"),
											$Request->input("AdminRole_CalendarAccess"),
											$Request->input("AdminRole_Status")
							);
			$oData = DB::select('call sp_admin_role_insert(?,?,?,?,?,?)',$oParam);

			$data = null;
			//  RESPONSE
			if ($oData[0]->Response_Success == 1) {
				#region HISTORY EXEC
				$this->history->insertBasic ($Request);
				$data = [ 'Id' => $oData[0]->Response_Id ];
				#endregion
			}
			
			$oResponse["Response_Status"]			= $oData[0]->Response_Status;
			$oResponse["Response_Code"]				= $oData[0]->Response_Code;
			$oResponse["Response_Domain"]			= $this->oDomain;
			$oResponse["Response_Message"]			= $oData[0]->Response_Message;
			$oResponse["Response_Data"]				= $data;
			$oResponse["Response_Error_Message"]	= $oData[0]->Response_Message;
			$oResponse["Response_Error_Reason"]		= $oData[0]->Response_Reason;
		}

		return ResponseBuilder::Response($oResponse);
		
	}

	
	public function Copy(Request $Request, $Id_AdminRole) {
        
        $oResponse = (new AdminRoleCopier(new MysqlAdminRoleRepository))
					  ->__invoke(new IdAdminRole($Id_AdminRole));

        $this->history->insert (
            [],
            $Request,
            $oResponse['Response_Data']['Id']
        );
        
        return ResponseBuilder::Response($oResponse);
	}


	public function Update(Request $Request){
		$oResponse  = array();
		$oValParams = array(    'Id_AdminRole'		=> 'required|int',
								'AdminRole_Name'	=> 'required|string|max:250',
								'AdminRole_Abrv'	=> 'required|string|max:4',
								'AdminRole_Color'	=> 'required|string|max:10',
								'AdminRole_Admin'	=> 'required|int',
								'AdminRole_CalendarAccess'	=> 'required|int',
								'AdminRole_Status'	=> 'required|int'
							);


		// VALIDANDO DATOS
		$oResponse = ParamsValidator::Validate_Request($Request,$oValParams,$this->oDomain);
		
		//  INGRESANDO DATOS
		if ($oResponse["Response_Code"] == 200) {
			
			#region HISTORY VARIABLES
			$this->history->obtainOld($Request->input('Id_AdminRole'));
			#endregion

			$oParam = array(    			$Request->input("Id_AdminRole"),
								Str::title( $Request->input("AdminRole_Name")),
								Str::upper( $Request->input("AdminRole_Abrv")),
								Str::upper( $Request->input("AdminRole_Color")),
											$Request->input("AdminRole_Admin"),
											$Request->input("AdminRole_CalendarAccess"),
											$Request->input("AdminRole_Status")
							);
			$oData = DB::select('call sp_admin_role_update(?,?,?,?,?,?,?)',$oParam);

			//  RESPONSE
			if ($oData[0]->Response_Success == 1) {
				#region HISTORY EXEC
				$this->history->updateBasic ( $Request->input("Id_AdminRole"), $Request );
				#endregion
			} else {
				
			}
			
			$oResponse["Response_Status"]			= $oData[0]->Response_Status;
			$oResponse["Response_Code"]				= $oData[0]->Response_Code;
			$oResponse["Response_Domain"]			= $this->oDomain;
			$oResponse["Response_Message"]			= $oData[0]->Response_Message;
			$oResponse["Response_Data"]				= null;
			$oResponse["Response_Error_Message"]	= $oData[0]->Response_Message;
			$oResponse["Response_Error_Reason"]		= $oData[0]->Response_Reason;
		}

		return ResponseBuilder::Response($oResponse);

	}
	public function Delete(Request $Request, $Id){
		$oResponse  = array();

		// VALIDANDO DATOS
		$oResponse = ParamsValidator::Validate_Id($Id,$this->oDomain);
		
		//  INGRESANDO DATOS
		if ($oResponse["Response_Code"] == 200) {
			
			#region HISTORY
			$this->history->obtainOld($Id);
			#endregion

			$oParam = array( $Id );
			$oData 	= DB::select('call sp_admin_role_delete(?)',$oParam);

			//  RESPONSE
			if ($oData[0]->Response_Success == 1) {
				#region HISTORY EXEC
				$this->history->deleteBasic ($Id, $Request);
				#endregion
			} else {
				
			}
			
			$oResponse["Response_Status"]			= $oData[0]->Response_Status;
			$oResponse["Response_Code"]				= $oData[0]->Response_Code;
			$oResponse["Response_Domain"]			= $this->oDomain;
			$oResponse["Response_Message"]			= $oData[0]->Response_Message;
			$oResponse["Response_Data"]				= null;
			$oResponse["Response_Error_Message"]	= $oData[0]->Response_Message;
			$oResponse["Response_Error_Reason"]		= $oData[0]->Response_Reason;
		}

		return ResponseBuilder::Response($oResponse);
	}
	public function Index(Request $Request, $Id){
		$oResponse  = array();

		// VALIDANDO DATOS
		$oResponse = ParamsValidator::Validate_Id($Id,$this->oDomain);
		
		//  INGRESANDO DATOS
		if ($oResponse["Response_Code"] == 200) {		
			$oParam = array( $Id );
			$oData 	= DB::select('call sp_admin_role_index(?)',$oParam);
			
			$oResponse["Response_Status"]			= 200;
			$oResponse["Response_Code"]				= 200;
			$oResponse["Response_Domain"]			= $this->oDomain;
			$oResponse["Response_Message"]			= $this->oDomain." Index";
			$oResponse["Response_Data"]				= $oData;
			$oResponse["Response_Error_Message"]	= "";
			$oResponse["Response_Error_Reason"]		= "";
		}

		return ResponseBuilder::Response($oResponse);
	}
	public function List(Request $Request, $Id){
		$oResponse  = array();

		$oResponse = ParamsValidator::Validate_Action($Id,$this->oDomain);

		if ($oResponse["Response_Code"] == 200) {
			$oParam = array( $Id );
			$oData  = DB::select('call sp_admin_role_list(?)',$oParam);

			$oResponse["Response_Status"]			= 200;
			$oResponse["Response_Code"]				= 200;
			$oResponse["Response_Domain"]			= $this->oDomain;
			$oResponse["Response_Message"]			= $this->oDomain." list ".$Id;
			$oResponse["Response_Data"]				= $oData;
			$oResponse["Response_Error_Message"]	= "";
			$oResponse["Response_Error_Reason"]		= "";
		}

		return ResponseBuilder::Response($oResponse);
	}
	
	public function Selector(Request $Request,$Id){
		if( is_numeric($Id) ){
			return SELF::Index( $Request,$Id );
		}else{
			return SELF::List( $Request,$Id );
		}
	}
	
}