<?php
namespace App\Http\Controllers\Owner;

use App\Http\Controllers\Controller;

use Illuminate\Http\Request;

use App\Http\Helpers\ResponseBuilder;
use App\Http\Helpers\ParamsValidator;
use App\Http\History\HistoryData;
use App\Http\History\HistoryTables;
use App\Modules\BookingTourPassenger\Application\BookingTourPassengerNotificationCreator;
use Illuminate\Support\Facades\DB;

class c_Booking_Tour_Passenger extends Controller
{
    private $oDomain;
    /** @var HistoryData */
    private $history;

    public function __construct()
    {
        $this->oDomain = "Booking Tour Passenger";
        $this->history = new HistoryData(HistoryTables::booking_tour_passenger);
    }
        
    public function Insert(Request $Request)
    {
        $oSucces    = true;
        $oResponse  = array();
        $oValParams = array(    'Id_Booking'        => 'required|int',
                                'Id_Package'        => 'required|int',
                                'Id_BookingTour'    => 'required|int',
                                'Group'             => 'required|int',
                                'BookingTours'      => 'required|string'
        );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request, $oValParams, $this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {
            $BookingTour = json_decode($Request->input("BookingTours"));

            for ($k = 0; $k < count($BookingTour); $k++) {
                for ($i = 0; $i < count($BookingTour[$k]->Pax); $i++) {
                    $oParam = [
                        $BookingTour[$k]->Id,
                        $BookingTour[$k]->Pax[$i]->Id,
                        count($BookingTour[$k]->Pax)
                    ];

                    $oData_TourPassenger    = DB::select('call sp_booking_tour_passenger_insert(?,?,?)', $oParam);
                    
                    $data = DB::select(
                        'CALL `sp_internal_booking_item_passenger_insert_required`(?,?, @sts, @cod, @msg)',
                        [ $oData_TourPassenger[0]->Response_Id, $BookingTour[$k]->Id ]
                    );
                    $oParam         = array( $BookingTour[$k]->Pax[$i]->Id, 1 );
                    $oData_Invoice  = DB::select('call sp_invoice_reload(?,?)', $oParam);
                    DB::select('call sp_passenger_check_info_pay(?)', [ $BookingTour[$k]->Pax[$i]->Id ]);
                    if ($oData_Invoice[0]->Response_Success == 0) {
                        $oSucces = false;
                        $oData = $oData_Invoice;
                        break 2;
                    }

                    if ($oData_TourPassenger[0]->Response_Success == 0) {
                        $oSucces = false;
                        $oData=$oData_TourPassenger;
                        break;
                    }

                    $this->history->insert(
                        [$oParam],
                        $Request,
                        $Request->input('Id_Booking')
                    );
                }
            }


            if ($oSucces == true) {
                $oData = array(
                    array(
                        "Response_Status"   => 200,
                        "Response_Code"     => 200,
                        "Response_Message"  => "Pasajeros guardados con éxito",
                        "Response_Reason"   => null
                    )
                );

                $oData = json_decode(json_encode($oData));
            }
            
            $oResponse["Response_Status"]           = $oData[0]->Response_Status;
            $oResponse["Response_Code"]             = $oData[0]->Response_Code;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $oData[0]->Response_Message;
            $oResponse["Response_Data"]             = null;
            $oResponse["Response_Error_Message"]    = $oData[0]->Response_Message;
            $oResponse["Response_Error_Reason"]     = $oData[0]->Response_Reason;
        }

        return ResponseBuilder::Response($oResponse);
    }
        
    public function Status(Request $Request)
    {
        $oResponse  = array();
        $oValParams = array(    'Id_BookingTourPassenger'       => 'required|int',
                                'BookingTourPassenger_Status'   => 'required|int'
                            );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request, $oValParams, $this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {
            $this->history->obtainOld($Request->input("Id_BookingTourPassenger"));

            $oParam = array(    $Request->input("Id_BookingTourPassenger"),
                                $Request->input("BookingTourPassenger_Status"),
                            );
            $oData  = DB::select('call sp_booking_tour_passenger_status(?,?)', $oParam);
            (new BookingTourPassengerNotificationCreator())
                ->__invoke($Request->input("Id_BookingTourPassenger"), $Request->header('Token'));
            

            //  RESPONSE
            if ($oData[0]->Response_Success == 1) {
                $this->history->update(
                    $this->findBooking($Request->input('Id_BookingTourPassenger')),
                    $this->history->oldValue,
                    $this->history->old($Request->input('Id_BookingTourPassenger')),
                    $Request
                );
            } else {
            }
            
            $oResponse["Response_Status"]           = $oData[0]->Response_Status;
            $oResponse["Response_Code"]             = $oData[0]->Response_Code;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $oData[0]->Response_Message;
            $oResponse["Response_Data"]             = null;
            $oResponse["Response_Error_Message"]    = $oData[0]->Response_Message;
            $oResponse["Response_Error_Reason"]     = $oData[0]->Response_Reason;
        }

        return ResponseBuilder::Response($oResponse);
    }

    public function Reportable(Request $Request)
    {
        $oResponse  = array();
        $oValParams = array(    'Id_BookingTourPassenger'           => 'required|int',
                                'BookingTourPassenger_Reportable'   => 'required|int'
                            );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request,$oValParams,$this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {
            $this->history->obtainOld($Request->input("Id_BookingTourPassenger"));
            
            $oParam = array(    $Request->input("Id_BookingTourPassenger"),
                                $Request->input("BookingTourPassenger_Reportable"),
                            );
            $oData  = DB::select('call sp_booking_tour_passenger_reportable(?,?)',$oParam);

            //  RESPONSE
            if ($oData[0]->Response_Success == 1) {
                $this->history->update (
                    $this->findBooking ($Request->input('Id_BookingTourPassenger')),
                    $this->history->oldValue,
                    $this->history->old ($Request->input('Id_BookingTourPassenger')),
                    $Request
                );
            }else{
                
            }
            
            $oResponse["Response_Status"]           = $oData[0]->Response_Status;
            $oResponse["Response_Code"]             = $oData[0]->Response_Code;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $oData[0]->Response_Message;
            $oResponse["Response_Data"]             = null;
            $oResponse["Response_Error_Message"]    = $oData[0]->Response_Message;
            $oResponse["Response_Error_Reason"]     = $oData[0]->Response_Reason;
        }

        return ResponseBuilder::Response($oResponse);
    }

    public function Hotel(Request $Request){
        $oResponse  = array();
        $oValParams = array(    'Id_BookingTourPassenger'       => 'required|int',
                                'BookingTourPassenger_Hotel'    => 'string'
                            );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request,$oValParams,$this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {

            $this->history->obtainOld ($Request->input('Id_BookingTourPassenger'));

            $oParam = array(    $Request->input("Id_BookingTourPassenger"),
                                $Request->input("BookingTourPassenger_Hotel"),
                            );
            $oData  = DB::select('call sp_booking_tour_passenger_hotel(?,?)',$oParam);

            //  RESPONSE
            if ($oData[0]->Response_Success == 1) {
                $this->history->update (
                    $this->findBooking ($Request->input('Id_BookingTourPassenger')),
                    $this->history->oldValue,
                    $this->history->old ($Request->input('Id_BookingTourPassenger')),
                    $Request
                );
            }else{
                
            }
            
            $oResponse["Response_Status"]           = $oData[0]->Response_Status;
            $oResponse["Response_Code"]             = $oData[0]->Response_Code;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $oData[0]->Response_Message;
            $oResponse["Response_Data"]             = null;
            $oResponse["Response_Error_Message"]    = $oData[0]->Response_Message;
            $oResponse["Response_Error_Reason"]     = $oData[0]->Response_Reason;
        }

        return ResponseBuilder::Response($oResponse);
    }

    public function Remark(Request $Request){
        $oResponse  = array();
        $oValParams = array(    'Id_BookingTourPassenger'       => 'required|int',
                                'BookingTourPassenger_Remark'   => 'required|string'
                            );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request,$oValParams,$this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {   
            $this->history->obtainOld($Request->input("Id_BookingTourPassenger"));  

            $oParam = array(    $Request->input("Id_BookingTourPassenger"),
                                $Request->input("BookingTourPassenger_Remark"),
                            );
            $oData  = DB::select('call sp_booking_tour_passenger_remark(?,?)',$oParam);

            //  RESPONSE
            if ($oData[0]->Response_Success == 1) {
                $this->history->update (
                    $this->findBooking ($Request->input('Id_BookingTourPassenger')),
                    $this->history->oldValue,
                    $this->history->old ($Request->input('Id_BookingTourPassenger')),
                    $Request
                );
            }else{
                
            }
            
            $oResponse["Response_Status"]           = $oData[0]->Response_Status;
            $oResponse["Response_Code"]             = $oData[0]->Response_Code;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $oData[0]->Response_Message;
            $oResponse["Response_Data"]             = null;
            $oResponse["Response_Error_Message"]    = $oData[0]->Response_Message;
            $oResponse["Response_Error_Reason"]     = $oData[0]->Response_Reason;
        }

        return ResponseBuilder::Response($oResponse);
    }

    private function findBooking ($Id_BookingTourPassenger) {
        $found = DB::select(
            'Call sp_booking_tour_passenger_index(?, ?)',
            [ $Id_BookingTourPassenger, config('var.PATH_PUBLIC') . config('var.USER_COUNTRY_THUMB') ]
        );

        return count ($found) ? $found[0]->Id_Booking : null;
    }
        
    public function Index(Request $Request, $Id) {
        $oResponse  = array();

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Id($Id, $this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {
            $oParam = array( $Id, config('var.PATH_PUBLIC') . config('var.USER_COUNTRY_THUMB') );
            $oData  = DB::select('call sp_booking_tour_passenger_index(?,?)', $oParam);
            
            $oResponse["Response_Status"]           = 200;
            $oResponse["Response_Code"]             = 200;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $this->oDomain." Index";
            $oResponse["Response_Data"]             = $oData;
            $oResponse["Response_Error_Message"]    = "";
            $oResponse["Response_Error_Reason"]     = "";
        }

        return ResponseBuilder::Response($oResponse);
    }

    public function List(Request $Request, $Id_BookingTour, $Id){
        $oResponse  = array();
        $oValParams = array(    'Id' => 'required|int'
                            );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request,$oValParams,$this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {

            $oResponse = ParamsValidator::Validate_Id($Id_BookingTour,$this->oDomain);

            if ($oResponse["Response_Code"] == 200) {
                
                $oResponse = ParamsValidator::Validate_Action($Id,$this->oDomain);

                $Path_Thumb     = config("var.PATH_PUBLIC").config("var.USER_COUNTRY_THUMB");
                if ($oResponse["Response_Code"] == 200) {
                    $oParam = array(    $Id,
                                        $Id_BookingTour,
                                        $Request->input("Id") ,
                                        $Path_Thumb
                                    );
                                    
                    $oData  = DB::select('call sp_booking_tour_passenger_list(?,?,?,?)',$oParam);

                    $oResponse["Response_Status"]           = 200;
                    $oResponse["Response_Code"]             = 200;
                    $oResponse["Response_Domain"]           = $this->oDomain;
                    $oResponse["Response_Message"]          = $this->oDomain." list";
                    $oResponse["Response_Data"]             = $oData;
                    $oResponse["Response_Error_Message"]    = "";
                    $oResponse["Response_Error_Reason"]     = "";
                }
            }
        }
        
        return ResponseBuilder::Response($oResponse);
    }

    public function Search(Request $Request)
    {
        $oResponse  = array();
        $oValParams = array(    'Id_Tour'   => 'required|int',
                                'Date'      => 'required|string'
                            );

        // VALIDANDO DATOS
        $oResponse = ParamsValidator::Validate_Request($Request, $oValParams, $this->oDomain);
        
        //  INGRESANDO DATOS
        if ($oResponse["Response_Code"] == 200) {
            $Path_Thumb     = config("var.PATH_PUBLIC") . config("var.USER_COUNTRY_THUMB");
            $oParam = [
                $Request->input("Id_Tour"),
                $Request->input("Date"),
                $Path_Thumb
            ];
            $oData  = DB::select('call sp_booking_tour_passenger_search(?,?,?)', $oParam);

            $oResponse["Response_Status"]           = 200;
            $oResponse["Response_Code"]             = 200;
            $oResponse["Response_Domain"]           = $this->oDomain;
            $oResponse["Response_Message"]          = $this->oDomain." search";
            $oResponse["Response_Data"]             = $oData;
            $oResponse["Response_Error_Message"]    = "";
            $oResponse["Response_Error_Reason"]     = "";
        }

        return ResponseBuilder::Response($oResponse);
    }
        
    public function Selector(Request $Request,$Id){
        if( is_numeric($Id) ){
            return SELF::Index( $Request,$Id );
        }else{
            return SELF::Search( $Request );
        }
    }

        
}